@@ -1,19 +1,35 @@
 import datetime
 import logging
 import re
+import sys
 import time
-import httplib2
+
 try:
-    import json as simplejson  # For Python 2.6
+    # For Python 3
+    from http.client import responses
+except ImportError:  # For Python 2.5-2.7
+    from httplib import responses
+except ImportError:  # For Python 2.4
+    from BaseHTTPServer import BaseHTTPRequestHandler
+    responses = dict([(k, v[0])
+                      for k, v in BaseHTTPRequestHandler.responses.items()])
+try:
+    import json as simplejson  # For Python 2.6+
 except ImportError:
     import simplejson
 from os import path
-from urlparse import (urlsplit, urlunsplit)
 try:
-    from urlparse import parse_qs
+    # For Python 3
+    from urllib.parse import (parse_qs, quote, urlencode, urlsplit, urlunsplit)
 except ImportError:
-    from cgi import parse_qs
-from urllib import urlencode, quote
+    from urlparse import (urlsplit, urlunsplit)
+    try:
+        from urlparse import parse_qs
+    except ImportError:
+        from cgi import parse_qs
+    from urllib import urlencode, quote
+
+import httplib2
 
 
 #: Hostname for API access
@@ -22,6 +38,24 @@ GITHUB_URL = "https://github.com"
 #: Logger for requests module
 LOGGER = logging.getLogger('github2.request')
 
+#: Whether github2 is using the system's certificates for SSL connections
+SYSTEM_CERTS = not httplib2.CA_CERTS.startswith(path.dirname(httplib2.__file__))
+if not SYSTEM_CERTS and sys.platform.startswith('linux'):
+    for cert_file in ['/etc/ssl/certs/ca-certificates.crt',
+                      '/etc/pki/tls/certs/ca-bundle.crt']:
+        if path.exists(cert_file):
+            httplib2.CA_CERTS = cert_file
+            SYSTEM_CERTS = True
+            break
+elif not SYSTEM_CERTS and sys.platform.startswith('freebsd'):
+    if path.exists('/usr/local/share/certs/ca-root-nss.crt'):
+        httplib2.CA_CERTS = '/usr/local/share/certs/ca-root-nss.crt'
+        SYSTEM_CERTS = True
+if SYSTEM_CERTS:
+    LOGGER.info('Using system certificates in %r', httplib2.CA_CERTS)
+else:
+    LOGGER.warning('Using bundled certificates for HTTPS connections')
+
 
 def charset_from_headers(headers):
     """Parse charset from headers
@@ -41,6 +75,25 @@ class GithubError(Exception):
     """An error occured when making a request to the Github API."""
 
 
+class HttpError(RuntimeError):
+    """A HTTP error occured when making a request to the Github API."""
+    def __init__(self, message, content, code):
+        """Create a HttpError exception
+
+        :param str message: Exception string
+        :param str content: Full content of HTTP request
+        :param int code: HTTP status code
+        """
+        self.args = (message, content, code)
+        self.message = message
+        self.content = content
+        self.code = code
+        if code and responses:
+            self.code_reason = responses[code]
+        else:
+            self.code_reason = ""
+
+
 class GithubRequest(object):
     github_url = GITHUB_URL
     url_format = "%(github_url)s/api/%(api_version)s/%(api_format)s"
@@ -70,25 +123,29 @@ class GithubRequest(object):
                 "api_version": self.api_version,
                 "api_format": self.api_format,
             }
-        digicert_ha_cert = path.join(path.dirname(path.abspath(__file__ )),
-                                     "DigiCert_High_Assurance_EV_Root_CA.crt")
         if proxy_host is None:
-            self._http = httplib2.Http(cache=cache, ca_certs=digicert_ha_cert)
+            self._http = httplib2.Http(cache=cache)
         else:
             proxy_info = httplib2.ProxyInfo(httplib2.socks.PROXY_TYPE_HTTP,
                                             proxy_host, proxy_port)
-            self._http = httplib2.Http(proxy_info=proxy_info, cache=cache,
-                                       ca_certs=digicert_ha_cert)
+            self._http = httplib2.Http(proxy_info=proxy_info, cache=cache)
+        if not SYSTEM_CERTS:
+            self._http.ca_certs = path.join(path.dirname(path.abspath(__file__)),
+                                            "DigiCert_High_Assurance_EV_Root_CA.crt")
 
     def encode_authentication_data(self, extra_post_data):
+        post_data = []
         if self.access_token:
-            post_data = {"access_token": self.access_token}
+            post_data.append(("access_token", self.access_token))
         elif self.username and self.api_token:
-            post_data = {"login": self.username,
-                         "token": self.api_token}
-        else:
-            post_data = {}
-        post_data.update(extra_post_data)
+            post_data.append(("login", self.username))
+            post_data.append(("token", self.api_token))
+        for key, value in extra_post_data.items():
+            if isinstance(value, list):
+                for elem in value:
+                    post_data.append((key, elem))
+            else:
+                post_data.append((key, value))
         return urlencode(post_data)
 
     def get(self, *path_components):
@@ -143,8 +200,9 @@ class GithubRequest(object):
             logging.debug("URL: %r POST_DATA: %r RESPONSE_TEXT: %r", url,
                           post_data, content)
         if response.status >= 400:
-            raise RuntimeError("unexpected response from github.com %d: %r" % (
-                               response.status, content))
+            raise HttpError("Unexpected response from github.com %d: %r"
+                            % (response.status, content), content,
+                            response.status)
         json = simplejson.loads(content.decode(charset_from_headers(response)))
         if json.get("error"):
             raise self.GithubError(json["error"][0]["error"])
